import pytest

from starknet_py.contract import (
    Contract,
    PreparedFunctionInvokeV1,
    PreparedFunctionInvokeV3,
)
from starknet_py.hash.selector import get_selector_from_name
from starknet_py.net.client_errors import ClientError
from starknet_py.net.client_models import Call, ResourceBounds
from starknet_py.net.models import InvokeV1, InvokeV3
from starknet_py.tests.e2e.fixtures.constants import (
    MAX_FEE,
    MAX_RESOURCE_BOUNDS,
    MAX_RESOURCE_BOUNDS_L1,
)


@pytest.mark.asyncio
async def test_prepare_and_invoke_v1(map_contract):
    prepared_invoke = map_contract.functions["put"].prepare_invoke_v1(
        key=1, value=2, max_fee=MAX_FEE
    )
    assert isinstance(prepared_invoke, PreparedFunctionInvokeV1)

    invocation = await prepared_invoke.invoke()
    assert isinstance(invocation.invoke_transaction, InvokeV1)
    assert invocation.invoke_transaction.max_fee == MAX_FEE


@pytest.mark.asyncio
async def test_prepare_and_invoke_v3(map_contract):
    prepared_invoke = map_contract.functions["put"].prepare_invoke_v3(
        key=1, value=2, l1_resource_bounds=MAX_RESOURCE_BOUNDS_L1
    )
    assert isinstance(prepared_invoke, PreparedFunctionInvokeV3)

    invocation = await prepared_invoke.invoke()
    assert isinstance(invocation.invoke_transaction, InvokeV3)
    assert invocation.invoke_transaction.resource_bounds == MAX_RESOURCE_BOUNDS


@pytest.mark.asyncio
async def test_invoke_v1(map_contract):
    invocation = await map_contract.functions["put"].invoke_v1(
        key=1, value=2, max_fee=MAX_FEE
    )
    assert isinstance(invocation.invoke_transaction, InvokeV1)
    assert invocation.invoke_transaction.max_fee == MAX_FEE


@pytest.mark.asyncio
async def test_invoke_v3(map_contract):
    invocation = await map_contract.functions["put"].invoke_v3(
        key=1, value=2, l1_resource_bounds=MAX_RESOURCE_BOUNDS_L1
    )
    assert isinstance(invocation.invoke_transaction, InvokeV3)
    assert invocation.invoke_transaction.resource_bounds == MAX_RESOURCE_BOUNDS


@pytest.mark.asyncio
async def test_auto_fee_estimation_v1(map_contract):
    prepared_invoke = map_contract.functions["put"].prepare_invoke_v1(key=1, value=2)
    assert isinstance(prepared_invoke, PreparedFunctionInvokeV1)

    invocation = await prepared_invoke.invoke(auto_estimate=True)
    assert isinstance(invocation.invoke_transaction, InvokeV1)
    assert invocation.invoke_transaction.max_fee is not None


@pytest.mark.asyncio
async def test_auto_fee_estimation_v3(map_contract):
    prepared_invoke = map_contract.functions["put"].prepare_invoke_v3(key=1, value=2)
    assert isinstance(prepared_invoke, PreparedFunctionInvokeV3)

    invocation = await prepared_invoke.invoke(auto_estimate=True)
    assert isinstance(invocation.invoke_transaction, InvokeV3)
    assert invocation.invoke_transaction.resource_bounds is not None


@pytest.mark.asyncio
async def test_throws_invoke_v1_without_max_fee(map_contract):
    error_message = "Argument max_fee must be specified when invoking a transaction."

    with pytest.raises(ValueError, match=error_message):
        await map_contract.functions["put"].invoke_v1(2, 3)


@pytest.mark.asyncio
async def test_throws_invoke_v3_without_resource_bounds(map_contract):
    error_message = (
        "One of arguments: "
        "l1_resource_bounds or auto_estimate must be specified when invoking a transaction."
    )

    with pytest.raises(ValueError, match=error_message):
        await map_contract.functions["put"].invoke_v3(2, 3)


@pytest.mark.asyncio
async def test_throws_prepared_invoke_v1_without_max_fee(map_contract):
    error_message = "Argument max_fee must be specified when invoking a transaction."

    prepared_invoke = map_contract.functions["put"].prepare_invoke_v1(2, 3)
    assert isinstance(prepared_invoke, PreparedFunctionInvokeV1)

    with pytest.raises(ValueError, match=error_message):
        await prepared_invoke.invoke()


@pytest.mark.asyncio
async def test_throws_prepared_invoke_v3_without_resource_bounds(map_contract):
    error_message = (
        "One of arguments: "
        "l1_resource_bounds or auto_estimate must be specified when invoking a transaction."
    )

    prepared_invoke = map_contract.functions["put"].prepare_invoke_v3(2, 3)
    assert isinstance(prepared_invoke, PreparedFunctionInvokeV3)

    with pytest.raises(ValueError, match=error_message):
        await prepared_invoke.invoke()


@pytest.mark.asyncio
async def test_throws_prepared_invoke_v1_with_max_fee_and_invoke_with_auto_estimate(
    map_contract,
):
    error_message = "Arguments max_fee and auto_estimate are mutually exclusive."

    invocation = map_contract.functions["put"].prepare_invoke_v1(
        key=2, value=3, max_fee=2000
    )
    with pytest.raises(ValueError, match=error_message):
        await invocation.invoke(auto_estimate=True)


@pytest.mark.asyncio
async def test_throws_when_invoke_v1_with_max_fee_and_auto_estimate(map_contract):
    error_message = "Arguments max_fee and auto_estimate are mutually exclusive."

    prepared_invoke = map_contract.functions["put"].prepare_invoke_v1(key=2, value=3)
    with pytest.raises(ValueError, match=error_message):
        await prepared_invoke.invoke(max_fee=10, auto_estimate=True)


@pytest.mark.asyncio
async def test_latest_max_fee_takes_precedence(map_contract):
    prepared_function = map_contract.functions["put"].prepare_invoke_v1(
        key=1, value=2, max_fee=MAX_FEE
    )
    invocation = await prepared_function.invoke(max_fee=MAX_FEE + 30)

    assert isinstance(invocation.invoke_transaction, InvokeV1)
    assert invocation.invoke_transaction.max_fee == MAX_FEE + 30


@pytest.mark.asyncio
async def test_latest_resource_bounds_take_precedence(map_contract):
    prepared_function = map_contract.functions["put"].prepare_invoke_v3(
        key=1, value=2, l1_resource_bounds=MAX_RESOURCE_BOUNDS_L1
    )

    updated_resource_bounds = ResourceBounds(
        max_amount=MAX_RESOURCE_BOUNDS_L1.max_amount + 100,
        max_price_per_unit=MAX_RESOURCE_BOUNDS_L1.max_price_per_unit + 200,
    )
    invocation = await prepared_function.invoke(
        l1_resource_bounds=updated_resource_bounds
    )

    assert isinstance(invocation.invoke_transaction, InvokeV3)
    assert (
        invocation.invoke_transaction.resource_bounds.l1_gas == updated_resource_bounds
    )
    assert (
        invocation.invoke_transaction.resource_bounds.l2_gas
        == ResourceBounds.init_with_zeros()
    )


@pytest.mark.asyncio
async def test_prepare_without_max_fee(map_contract):
    prepared_invoke = map_contract.functions["put"].prepare_invoke_v1(key=1, value=2)

    assert prepared_invoke.max_fee is None


@pytest.mark.asyncio
@pytest.mark.parametrize("key, value", ((2, 13), (412312, 32134), (12345, 3567)))
async def test_invoke_v1_and_call(key, value, map_contract):
    invocation = await map_contract.functions["put"].invoke_v1(
        key, value, max_fee=MAX_FEE
    )
    await invocation.wait_for_acceptance()
    assert isinstance(invocation.invoke_transaction, InvokeV1)

    (response,) = await map_contract.functions["get"].call(key)
    assert response == value


@pytest.mark.asyncio
async def test_call_uninitialized_contract(client):
    with pytest.raises(ClientError, match="Contract not found"):
        await client.call_contract(
            Call(
                to_addr=1,
                selector=get_selector_from_name("get_nonce"),
                calldata=[],
            ),
            block_hash="latest",
        )


@pytest.mark.asyncio
async def test_wait_for_tx(client, map_contract):
    transaction = await map_contract.functions["put"].invoke_v1(
        key=10, value=20, max_fee=MAX_FEE
    )
    await client.wait_for_tx(transaction.hash)


@pytest.mark.asyncio
async def test_error_when_prepare_without_account(client, map_contract):
    contract = await Contract.from_address(map_contract.address, client)

    with pytest.raises(
        ValueError,
        match="Contract instance was created without providing an Account.",
    ):
        contract.functions["put"].prepare_invoke_v1(key=10, value=10)


@pytest.mark.asyncio
async def test_error_when_invoke_without_account(client, map_contract):
    contract = await Contract.from_address(map_contract.address, client)

    with pytest.raises(
        ValueError,
        match="Contract instance was created without providing an Account.",
    ):
        await contract.functions["put"].invoke_v1(key=10, value=10)



# pylint: disable=redefined-outer-name
from typing import Tuple

import pytest
import pytest_asyncio

from starknet_py.contract import Contract
from starknet_py.net.account.account import Account
from starknet_py.net.full_node_client import FullNodeClient
from starknet_py.net.models.transaction import DeployAccountV1
from starknet_py.net.udc_deployer.deployer import Deployer
from starknet_py.tests.e2e.client.fixtures.prepare_net_for_gateway_test import (
    PreparedNetworkData,
)
from starknet_py.tests.e2e.fixtures.constants import MAX_FEE
from starknet_py.tests.e2e.utils import (
    get_deploy_account_details,
    get_deploy_account_transaction,
)


@pytest_asyncio.fixture(scope="package")
async def deploy_account_transaction(
    account_with_validate_deploy_class_hash: int,
    eth_fee_contract: Contract,
    strk_fee_contract: Contract,
    devnet,
) -> DeployAccountV1:
    """
    Returns a DeployAccount transaction
    """

    address, key_pair, salt, class_hash = await get_deploy_account_details(
        class_hash=account_with_validate_deploy_class_hash,
        eth_fee_contract=eth_fee_contract,
        strk_fee_contract=strk_fee_contract,
    )
    return await get_deploy_account_transaction(
        address=address,
        key_pair=key_pair,
        class_hash=class_hash,
        salt=salt,
        client=FullNodeClient(devnet),
    )


@pytest.fixture(scope="package")
def deploy_account_transaction_hash(
    prepare_network: Tuple[str, PreparedNetworkData]
) -> int:
    """
    Returns hash of deploy account transaction
    """
    _, prepared_data = prepare_network
    return prepared_data.deploy_account_transaction_hash


@pytest.fixture(scope="package")
def block_with_deploy_account_number(
    prepare_network: Tuple[str, PreparedNetworkData]
) -> int:
    """
    Returns number of the block with deploy account transaction
    """
    _, prepared_data = prepare_network
    return prepared_data.block_with_deploy_account_number


@pytest_asyncio.fixture(scope="package")
async def hello_starknet_deploy_transaction_address(
    account: Account, hello_starknet_class_hash
) -> int:
    deployer = Deployer()
    contract_deployment = deployer.create_contract_deployment_raw(
        class_hash=hello_starknet_class_hash
    )
    deploy_invoke_transaction = await account.sign_invoke_v1(
        calls=contract_deployment.call, max_fee=MAX_FEE
    )
    resp = await account.client.send_transaction(deploy_invoke_transaction)
    await account.client.wait_for_tx(resp.transaction_hash)
    return contract_deployment.address


@pytest_asyncio.fixture(scope="package")
async def block_with_declare_v2_number(hello_starknet_tx_hash: int, client) -> int:
    """
    Returns number of the block with DeclareV2 transaction
    """
    declare_v2_receipt = await client.get_transaction_receipt(hello_starknet_tx_hash)
    return declare_v2_receipt.block_number



# fmt: off
import pytest

from starknet_py.common import create_casm_class
from starknet_py.hash.casm_class_hash import compute_casm_class_hash
from starknet_py.tests.e2e.fixtures.constants import PRECOMPILED_CONTRACTS_DIR
from starknet_py.tests.e2e.fixtures.misc import (
    ContractVersion,
    load_contract,
    read_contract,
)


@pytest.mark.parametrize(
    "contract, expected_casm_class_hash",
    [
        ("Account", 0x108977ab61715437fc7097b6499b3cf9491361eb6a8ce6df6c8536b7feec508),
        ("ERC20", 0x5adc857416202a5902c01168542e188c3aa6380f57c911ae98cf20bc52be367),
        ("HelloStarknet", 0x6ff9f7df06da94198ee535f41b214dce0b8bafbdb45e6c6b09d4b3b693b1f17),
        ("TestContract", 0x2193add92c182c9236f0c156f11dc4f18d5a78fd9b763a3c0f4a1d3bd8b87d4),
        ("TokenBridge", 0x41d26534c7ca29e212ae48acfb9f86f69a9624977c979697c15f587fa95204),
    ],
)
def test_compute_casm_class_hash(contract, expected_casm_class_hash):
    casm_contract_class_str = load_contract(
        contract, version=ContractVersion.V2
    )['casm']

    casm_class = create_casm_class(casm_contract_class_str)
    casm_class_hash = compute_casm_class_hash(casm_class)
    assert casm_class_hash == expected_casm_class_hash

@pytest.mark.parametrize(
    "casm_contract_class_source, expected_casm_class_hash",
    [
        ("minimal_contract_compiled_v2_1.casm",
         0x186f6c4ca3af40dbcbf3f08f828ab0ee072938aaaedccc74ef3b9840cbd9fb3),
        ("minimal_contract_compiled_v2_5_4.casm",
         0x1d055a90aa90db474fa08a931d5e63753c6f762fa3e9597b26c8d4b003a2de6),
        ("starknet_contract_v2_6.casm", 0x603dd72504d8b0bc54df4f1102fdcf87fc3b2b94750a9083a5876913eec08e4),
    ],
)
def test_precompiled_compute_casm_class_hash(casm_contract_class_source, expected_casm_class_hash):
    casm_contract_class_str = read_contract(
        casm_contract_class_source, directory=PRECOMPILED_CONTRACTS_DIR
    )

    casm_class = create_casm_class(casm_contract_class_str)
    casm_class_hash = compute_casm_class_hash(casm_class)
    assert casm_class_hash == expected_casm_class_hash



# pylint: disable=unused-variable
import pytest

from starknet_py.contract import Contract
from starknet_py.net.client_models import ResourceBounds


@pytest.mark.asyncio
async def test_invoke(map_contract: Contract):
    prepared_function_call = map_contract.functions["put"].prepare_invoke_v3(
        key=10, value=20
    )
    l1_resource_bounds = ResourceBounds(max_amount=5000, max_price_per_unit=int(1e12))
    # docs-start: invoke
    invoke_result = await prepared_function_call.invoke(
        l1_resource_bounds=ResourceBounds(max_amount=5000, max_price_per_unit=int(1e12))
    )
    # docs-end: invoke
    prepared_function_call.l1_resource_bounds = None
    # docs-start: invoke
    # l1_resource_bounds can be estimated automatically
    invoke_result = await prepared_function_call.invoke(auto_estimate=True)
    # or if l1_resource_bounds was specified in prepared_function_call
    # docs-end: invoke
    prepared_function_call.l1_resource_bounds = l1_resource_bounds
    # docs-start: invoke
    invoke_result = await prepared_function_call.invoke()
    # docs-end: invoke



import pytest

from starknet_py.net.full_node_client import FullNodeClient


@pytest.fixture(name="client", scope="package")
def create_full_node_client(devnet) -> FullNodeClient:
    return FullNodeClient(node_url=devnet + "/rpc")


@pytest.fixture(name="client_fork_mode", scope="package")
def create_full_node_client_fork_mode(devnet_fork_mode) -> FullNodeClient:
    return FullNodeClient(node_url=devnet_fork_mode + "/rpc")



import pytest

from starknet_py.contract import Contract
from starknet_py.tests.e2e.fixtures.constants import MAX_FEE, MAX_RESOURCE_BOUNDS_L1
from starknet_py.tests.e2e.fixtures.misc import load_contract


@pytest.mark.asyncio
async def test_throws_when_cairo1_without_compiled_contract_casm_and_class_hash(
    account,
):
    error_message = (
        "For Cairo 1.0 contracts, either the 'compiled_class_hash' or the 'compiled_contract_casm' "
        "argument must be provided."
    )
    compiled_contract = load_contract("Map")["sierra"]

    with pytest.raises(ValueError, match=error_message):
        await Contract.declare_v2(
            account, compiled_contract=compiled_contract, max_fee=MAX_FEE
        )

    with pytest.raises(ValueError, match=error_message):
        await Contract.declare_v3(
            account,
            compiled_contract=compiled_contract,
            l1_resource_bounds=MAX_RESOURCE_BOUNDS_L1,
        )



# pylint: disable=unused-variable
import pytest

from starknet_py.contract import Contract


@pytest.mark.asyncio
async def test_invoke(map_contract: Contract):
    prepared_function_call = map_contract.functions["put"].prepare_invoke_v1(
        key=10, value=20
    )
    # docs-start: invoke
    invoke_result = await prepared_function_call.invoke(max_fee=int(1e15))
    # docs-end: invoke
    prepared_function_call.max_fee = None
    # docs-start: invoke
    # max_fee can be estimated automatically
    invoke_result = await prepared_function_call.invoke(auto_estimate=True)
    # or if max_fee was specified in prepared_function_call
    # docs-end: invoke
    prepared_function_call.max_fee = int(1e15)
    # docs-start: invoke
    invoke_result = await prepared_function_call.invoke()
    # docs-end: invoke



import datetime

import pytest

from starknet_py.constants import ANY_CALLER, OutsideExecutionInterfaceID
from starknet_py.hash.selector import get_selector_from_name
from starknet_py.net.account.account import BaseAccount
from starknet_py.net.client_models import Call, OutsideExecutionTimeBounds
from starknet_py.tests.e2e.fixtures.constants import MAX_FEE
from starknet_py.transaction_errors import TransactionRevertedError


@pytest.mark.asyncio
async def test_argent_account_outside_execution_compatibility(
    argent_account: BaseAccount,
):
    result = await argent_account.supports_interface(OutsideExecutionInterfaceID.V1)
    assert result is True
    result = await argent_account.supports_interface(OutsideExecutionInterfaceID.V2)
    assert result is False


# TODO (#1546): Remove skip mark
@pytest.mark.skip
@pytest.mark.asyncio
async def test_account_outside_execution_any_caller(
    argent_account: BaseAccount,
    map_contract,
):

    assert any(
        [
            await argent_account.supports_interface(OutsideExecutionInterfaceID.V1),
            await argent_account.supports_interface(OutsideExecutionInterfaceID.V2),
        ]
    )

    put_call = Call(
        to_addr=map_contract.address,
        selector=get_selector_from_name("put"),
        calldata=[20, 20],
    )

    call = await argent_account.sign_outside_execution_call(
        calls=[
            put_call,
        ],
        execution_time_bounds=OutsideExecutionTimeBounds(
            execute_after=datetime.datetime.now() - datetime.timedelta(hours=1),
            execute_before=datetime.datetime.now() + datetime.timedelta(hours=1),
        ),
        caller=ANY_CALLER,
    )

    tx = await argent_account.execute_v1(calls=[call], max_fee=MAX_FEE)
    await argent_account.client.wait_for_tx(tx.transaction_hash)


@pytest.mark.asyncio
async def test_account_outside_execution_for_invalid_caller(
    argent_account: BaseAccount,
    account: BaseAccount,
    map_contract,
):
    assert any(
        [
            await argent_account.supports_interface(OutsideExecutionInterfaceID.V1),
            await argent_account.supports_interface(OutsideExecutionInterfaceID.V2),
        ]
    )

    put_call = Call(
        to_addr=map_contract.address,
        selector=get_selector_from_name("put"),
        calldata=[20, 20],
    )

    call = await argent_account.sign_outside_execution_call(
        calls=[
            put_call,
        ],
        execution_time_bounds=OutsideExecutionTimeBounds(
            execute_after=datetime.datetime.now() - datetime.timedelta(hours=1),
            execute_before=datetime.datetime.now() + datetime.timedelta(hours=1),
        ),
        caller=account.address,
    )

    tx = await argent_account.execute_v1(calls=[call], max_fee=MAX_FEE)

    with pytest.raises(TransactionRevertedError) as err:
        await argent_account.client.wait_for_tx(tx.transaction_hash)

    assert "argent/invalid-caller" in err.value.message


# TODO (#1546): Remove skip mark
@pytest.mark.skip
@pytest.mark.asyncio
async def test_account_outside_execution_for_impossible_time_bounds(
    argent_account: BaseAccount,
    map_contract,
):

    assert any(
        [
            await argent_account.supports_interface(OutsideExecutionInterfaceID.V1),
            await argent_account.supports_interface(OutsideExecutionInterfaceID.V2),
        ]
    )

    put_call = Call(
        to_addr=map_contract.address,
        selector=get_selector_from_name("put"),
        calldata=[20, 20],
    )

    call = await argent_account.sign_outside_execution_call(
        calls=[put_call],
        execution_time_bounds=OutsideExecutionTimeBounds(
            execute_after=datetime.datetime.now() - datetime.timedelta(days=10),
            execute_before=datetime.datetime.now() - datetime.timedelta(days=9),
        ),
        caller=ANY_CALLER,
    )

    tx = await argent_account.execute_v1(calls=[call], max_fee=MAX_FEE)

    with pytest.raises(TransactionRevertedError) as err:
        await argent_account.client.wait_for_tx(tx.transaction_hash)

    assert "argent/invalid-timestamp" in err.value.message



# pylint: disable=redefined-outer-name

import json
import sys
from enum import Enum
from pathlib import Path
from typing import Optional

import pytest

from starknet_py.net.full_node_client import FullNodeClient
from starknet_py.net.models.typed_data import TypedDataDict
from starknet_py.tests.e2e.fixtures.constants import (
    CONTRACTS_V1_ARTIFACTS_MAP,
    CONTRACTS_V1_COMPILED,
    CONTRACTS_V2_ARTIFACTS_MAP,
    CONTRACTS_V2_COMPILED,
    TYPED_DATA_DIR,
)


def pytest_addoption(parser):
    parser.addoption(
        "--contract_dir",
        action="store",
        default="",
        help="Contract directory: possible 'v1', 'v2'",
    )


@pytest.fixture(
    params=[
        "typed_data_rev_0_example.json",
        "typed_data_rev_0_felt_array_example.json",
        "typed_data_rev_0_long_string_example.json",
        "typed_data_rev_0_struct_array_example.json",
        "typed_data_rev_1_example.json",
    ],
)
def typed_data(request) -> TypedDataDict:
    """
    Returns TypedData dictionary example.
    """
    file_name = getattr(request, "param")
    file_path = TYPED_DATA_DIR / file_name

    with open(file_path, "r", encoding="utf-8") as file:
        typed_data = json.load(file)

    return typed_data


@pytest.fixture(name="get_tx_receipt_path", scope="package")
def get_tx_receipt_full_node_client():
    return f"{FullNodeClient.__module__}.FullNodeClient.get_transaction_receipt"


@pytest.fixture(name="get_tx_status_path", scope="package")
def get_tx_status_full_node_client():
    return f"{FullNodeClient.__module__}.FullNodeClient.get_transaction_status"


class ContractVersion(Enum):
    V1 = "V1"
    V2 = "V2"


class UnknownArtifacts(BaseException):
    pass


def load_contract(contract_name: str, version: Optional[ContractVersion] = None):
    if version is None:
        if "--contract_dir=v1" in sys.argv:
            version = ContractVersion.V1
        if "--contract_dir=v2" in sys.argv:
            version = ContractVersion.V2

    if version == ContractVersion.V1:
        artifacts_map_path = CONTRACTS_V1_ARTIFACTS_MAP
        directory = CONTRACTS_V1_COMPILED
    else:
        artifacts_map_path = CONTRACTS_V2_ARTIFACTS_MAP
        directory = CONTRACTS_V2_COMPILED

    artifacts_map = json.loads((artifacts_map_path).read_text("utf-8"))

    artifact_file_names = next(
        (
            item["artifacts"]
            for item in artifacts_map["contracts"]
            if item["contract_name"] == contract_name
        ),
        None,
    )

    if not isinstance(artifact_file_names, dict):  # pyright: ignore
        raise UnknownArtifacts(f"Artifacts for contract {contract_name} not found")

    sierra = (directory / artifact_file_names["sierra"]).read_text("utf-8")
    casm = (directory / artifact_file_names["casm"]).read_text("utf-8")

    return {"casm": casm, "sierra": sierra}


def read_contract(file_name: str, *, directory: Path) -> str:
    """
    Return contents of file_name from directory.
    """
    return (directory / file_name).read_text("utf-8")



import pytest

from starknet_py.common import create_sierra_compiled_contract
from starknet_py.hash.sierra_class_hash import compute_sierra_class_hash
from starknet_py.tests.e2e.fixtures.misc import ContractVersion, load_contract


@pytest.mark.parametrize(
    "contract_name, expected_class_hash",
    # fmt: off
    [
        ("Account", 0x183078afce57a1d33b948ea6cd9ab0769dd08ca93a6afe4c23637b08aa893c1),
        ("ERC20", 0x746248ba570006607113ae3f4dbb4130e81233fb818d15329c6a4aaccf94812),
        ("HelloStarknet", 0x224518978adb773cfd4862a894e9d333192fbd24bc83841dc7d4167c09b89c5),
        ("MinimalContract", 0x6fb1efd745d57b60023c6dc3209227e5e54d44fa16e0ae75cc03e1a7f3da08a),
        ("TestContract", 0x3adac8a417b176d27e11b420aa1063b07a6b54bbb21091ad77b2a9156af7a3b),
        ("TokenBridge", 0x3d138e923f01b7ed1bb82b9b4e7f6df64e0c429faf8b27539addc71c1407237),
    ],
    # fmt: on
)
def test_compute_sierra_class_hash(contract_name, expected_class_hash):
    sierra_contract_class_str = load_contract(
        contract_name=contract_name, version=ContractVersion.V2
    )["sierra"]

    sierra_contract_class = create_sierra_compiled_contract(sierra_contract_class_str)
    class_hash = compute_sierra_class_hash(sierra_contract_class)

    assert class_hash == expected_class_hash



# pylint: disable=unused-variable
import pytest

from starknet_py.contract import Contract, ContractFunction
from starknet_py.net.client_models import ResourceBounds


def test_prepare_invoke_v1(map_contract: Contract):
    # docs-start: prepare_invoke_v1
    prepared_function_call = map_contract.functions["put"].prepare_invoke_v1(
        key=10, value=20
    )
    # docs-end: prepare_invoke_v1


def test_prepare_invoke_v3(map_contract: Contract):
    # docs-start: prepare_invoke_v3
    prepared_function_call = map_contract.functions["put"].prepare_invoke_v3(
        key=10, value=20
    )
    # docs-end: prepare_invoke_v3


@pytest.mark.asyncio
async def test_call(map_contract: Contract):
    # docs-start: call
    call_result = map_contract.functions["get"].call(key=10)
    # or when call has to be done at specific block
    call_result = map_contract.functions["get"].call(key=10, block_number="latest")
    # docs-end: call


def test_invoke_v1(map_contract: Contract):
    # docs-start: invoke_v1
    invoke_result = map_contract.functions["put"].invoke_v1(
        key=10, value=20, max_fee=int(1e15)
    )
    # docs-end: invoke_v1


def test_invoke_v3(map_contract: Contract):
    # docs-start: invoke_v3
    invoke_result = map_contract.functions["put"].invoke_v3(
        key=10,
        value=20,
        l1_resource_bounds=ResourceBounds(
            max_amount=5000, max_price_per_unit=int(1e12)
        ),
    )
    # docs-end: invoke_v3


def test_get_selector():
    # docs-start: get_selector
    selector = ContractFunction.get_selector("get")
    # docs-end: get_selector
